@@ -2,12 +2,12 @@ extern crate polytype;
 extern crate programinduction;
 extern crate rayon;
 extern crate serde;
-extern crate serde_json;
 #[macro_use]
 extern crate serde_derive;
+extern crate serde_json;
 
 use std::f64;
-use polytype::Type;
+use polytype::TypeSchema;
 use programinduction::{lambda, ECFrontier, Task};
 use rayon::prelude::*;
 
@@ -79,7 +79,7 @@ impl From<ExternalCompressionInput> for CompressionInput {
             .map(|p| {
                 (
                     p.name,
-                    Type::parse(&p.tp).expect("invalid primitive type"),
+                    TypeSchema::parse(&p.tp).expect("invalid primitive type"),
                     p.logp,
                 )
             })
@@ -90,15 +90,11 @@ impl From<ExternalCompressionInput> for CompressionInput {
             invented: vec![],
             variable_logprob,
         };
-        let invented = eci.inventions
-            .into_par_iter()
-            .map(|inv| {
-                let expr = dsl.parse(&inv.expression).expect("invalid invention");
-                let tp = dsl.infer(&expr).expect("invalid invention type");
-                (expr, tp, inv.logp)
-            })
-            .collect();
-        dsl.invented = invented;
+        for inv in eci.inventions {
+            let expr = dsl.parse(&inv.expression).expect("invalid invention");
+            let tp = dsl.infer(&expr).expect("invalid invention type");
+            dsl.invented.push((expr, tp, inv.logp))
+        }
         let params = lambda::CompressionParams {
             pseudocounts: eci.params.pseudocounts,
             topk: eci.params.topk,
@@ -109,7 +105,7 @@ impl From<ExternalCompressionInput> for CompressionInput {
         let (tasks, frontiers) = eci.frontiers
             .into_par_iter()
             .map(|f| {
-                let tp = Type::parse(&f.task_tp).expect("invalid task type");
+                let tp = TypeSchema::parse(&f.task_tp).expect("invalid task type");
                 let task = Task {
                     oracle: Box::new(noop_oracle),
                     observation: (),
